"use strict";
var JsonApiModule_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonApiModule = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const utils_1 = require("./utils");
const constants_1 = require("./constants");
const modules_1 = require("./modules");
const interceptors_1 = require("./modules/mixin/interceptors");
const service_1 = require("./modules/mixin/service");
let JsonApiModule = JsonApiModule_1 = class JsonApiModule {
    static forRoot(module, options) {
        const prepareOptions = (0, utils_1.prepareConfig)(options);
        prepareOptions.providers.push(interceptors_1.ErrorInterceptors, service_1.ErrorFormatService, interceptors_1.LogTimeInterceptors);
        const commonOrmModule = module.forRoot(prepareOptions);
        if (!commonOrmModule.providers ||
            !commonOrmModule.providers.find((i) => 'provide' in i && i.provide === constants_1.ENTITY_PARAM_MAP))
            throw new Error(`The module ${module.name} should be provide ${constants_1.ENTITY_PARAM_MAP.description}`);
        const entitiesModules = prepareOptions.entities.map((entityItem) => modules_1.MixinModule.forRoot({
            entity: entityItem,
            imports: [commonOrmModule, ...prepareOptions.imports],
            ormModule: module,
            controller: (0, utils_1.getController)(entityItem, prepareOptions.controllers),
            config: prepareOptions,
        }));
        const atomicOperation = prepareOptions.options.operationUrl
            ? modules_1.AtomicOperationModule.forRoot(prepareOptions.options.operationUrl, prepareOptions.entities, entitiesModules, commonOrmModule)
            : [];
        return {
            module: JsonApiModule_1,
            imports: [...entitiesModules, ...atomicOperation],
        };
    }
};
exports.JsonApiModule = JsonApiModule;
exports.JsonApiModule = JsonApiModule = JsonApiModule_1 = tslib_1.__decorate([
    (0, common_1.Module)({})
], JsonApiModule);
//# sourceMappingURL=json-api.module.js.map