"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MixinModule = void 0;
const core_1 = require("@nestjs/core");
const helpers_1 = require("./helpers");
const constants_1 = require("../../constants");
const bind_controller_1 = require("./helpers/bind-controller");
const service_1 = require("./service");
const swagger_1 = require("./swagger");
const factory_1 = require("./factory");
class MixinModule {
    static forRoot(mixinOptions) {
        const { entity, controller, imports, ormModule, config } = mixinOptions;
        const controllerClass = (0, helpers_1.createController)(entity, controller);
        const decoratorOptions = Reflect.getMetadata(constants_1.JSON_API_DECORATOR_OPTIONS, controllerClass) || {};
        const moduleConfig = {
            ...config.options,
            ...decoratorOptions,
        };
        (0, bind_controller_1.bindController)(controllerClass, entity, moduleConfig);
        const optionProvider = {
            provide: constants_1.CONTROLLER_OPTIONS_TOKEN,
            useValue: moduleConfig,
        };
        const currentEntityProvider = {
            provide: constants_1.CURRENT_ENTITY,
            useValue: entity,
        };
        return {
            module: (0, helpers_1.nameIt)((0, helpers_1.getProviderName)(entity, constants_1.JSON_API_MODULE_POSTFIX), MixinModule),
            controllers: [controllerClass],
            providers: [
                optionProvider,
                currentEntityProvider,
                service_1.EntityParamMapService,
                service_1.JsonApiTransformerService,
                swagger_1.SwaggerBindService,
                (0, factory_1.ZodInputQuerySchema)(),
                (0, factory_1.ZodPostSchema)(),
                (0, factory_1.ZodQuerySchema)(),
                (0, factory_1.ZodPatchSchema)(),
                factory_1.ZodInputPatchRelationshipSchema,
                factory_1.ZodInputPostRelationshipSchema,
                ...ormModule.getUtilProviders(entity),
            ],
            imports: [...imports, core_1.DiscoveryModule],
        };
    }
}
exports.MixinModule = MixinModule;
//# sourceMappingURL=mixin.module.js.map