"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.shapeForArray = shapeForArray;
exports.zodFilterInputQuery = zodFilterInputQuery;
const json_api_nestjs_shared_1 = require("@klerick/json-api-nestjs-shared");
const zod_1 = require("zod");
const zod_utils_1 = require("../zod-utils");
const arrayOp = {
    [json_api_nestjs_shared_1.FilterOperand.in]: true,
    [json_api_nestjs_shared_1.FilterOperand.nin]: true,
    [json_api_nestjs_shared_1.FilterOperand.some]: true,
};
function convertToFilterObject(value) {
    if ((0, json_api_nestjs_shared_1.isString)(value)) {
        return {
            [json_api_nestjs_shared_1.FilterOperand.eq]: value,
        };
    }
    else {
        return Object.entries(value).reduce((acum, [op, filed]) => {
            if (op in arrayOp) {
                acum[op] = ((0, json_api_nestjs_shared_1.isString)(filed) ? filed.split(',') : []).filter((i) => !!i);
            }
            else {
                acum[op] = filed;
            }
            return acum;
        }, {});
    }
}
function getZodRulesForFilterOperator() {
    const filterConditional = zod_1.z
        .union([zod_1.z.string().refine((0, zod_utils_1.stringLongerThan)()), zod_1.z.null(), zod_1.z.number()])
        .transform((r) => `${r}`);
    const conditional = zod_1.z
        .object(json_api_nestjs_shared_1.ObjectTyped.values(json_api_nestjs_shared_1.FilterOperand).reduce((acum, item) => {
        acum[item] = filterConditional;
        return acum;
    }, {}))
        .strict()
        .partial()
        .refine((0, zod_utils_1.oneOf)(Object.values(json_api_nestjs_shared_1.FilterOperand)), {
        message: `Must have one of: "${Object.values(json_api_nestjs_shared_1.FilterOperand).join('","')}"`,
    });
    return zod_1.z.union([filterConditional, conditional]).optional();
}
function getZodRulesForRelation() {
    return zod_1.z
        .union([
        zod_1.z
            .object({
            [json_api_nestjs_shared_1.FilterOperand.eq]: zod_1.z
                .union([zod_1.z.literal('null'), zod_1.z.null()])
                .transform(() => null),
        })
            .strict(),
        zod_1.z
            .object({
            [json_api_nestjs_shared_1.FilterOperand.ne]: zod_1.z
                .union([zod_1.z.literal('null'), zod_1.z.null()])
                .transform(() => null),
        })
            .strict(),
    ])
        .optional();
}
function getTupleConcatRelationFields(relationList) {
    const result = [];
    for (const [key, val] of json_api_nestjs_shared_1.ObjectTyped.entries(relationList)) {
        const relName = key.toString();
        for (const v of val) {
            result.push(`${relName}.${v}`);
        }
    }
    return result;
}
function shapeForArray(fields, zodSchema) {
    return fields.reduce((acum, item) => ({
        ...acum,
        [item]: zodSchema,
    }), {});
}
function zodFilterInputQuery(entityParamMapService) {
    const target = zod_1.z.object(shapeForArray(entityParamMapService.entityParaMap.props, getZodRulesForFilterOperator()));
    const relations = zod_1.z.object(shapeForArray(entityParamMapService.entityParaMap.relations, getZodRulesForRelation()));
    const relationList = (0, zod_utils_1.getRelationProps)(entityParamMapService);
    const relationFields = zod_1.z.object(shapeForArray(getTupleConcatRelationFields(relationList), getZodRulesForFilterOperator()));
    return target
        .merge(relations)
        .merge(relationFields)
        .optional()
        .transform((data) => {
        if (!data) {
            return {
                relation: null,
                target: null,
            };
        }
        return Object.entries(data).reduce((acum, [field, value]) => {
            const objectOperand = convertToFilterObject(value);
            if (Object.keys(objectOperand).length === 0) {
                return acum;
            }
            if (field.indexOf('.') > -1) {
                const [relation, fieldRelation] = field.split('.');
                acum['relation'] = !acum['relation'] ? {} : acum['relation'];
                acum['relation'][relation] = acum['relation'][relation] || {};
                acum['relation'][relation][fieldRelation] = objectOperand;
            }
            else {
                acum['target'] = !acum['target'] ? {} : acum['target'];
                acum['target'][field] = objectOperand;
            }
            return acum;
        }, { relation: null, target: null });
    });
}
//# sourceMappingURL=filter.js.map