"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.zodFieldsQuery = zodFieldsQuery;
const json_api_nestjs_shared_1 = require("@klerick/json-api-nestjs-shared");
const zod_1 = require("zod");
const zod_utils_1 = require("../zod-utils");
function getZodFieldRule(fields) {
    const fieldsProps = fields;
    return zod_1.z
        .enum(fieldsProps)
        .array()
        .nonempty()
        .refine((0, zod_utils_1.uniqueArray)(), {
        message: 'Field should be unique',
    })
        .optional();
}
function getShapeFields(entityParamMapService) {
    const relationList = (0, zod_utils_1.getRelationProps)(entityParamMapService);
    const fieldShape = json_api_nestjs_shared_1.ObjectTyped.entries(relationList).reduce((acum, [key, val]) => ({
        ...acum,
        [key]: getZodFieldRule(val),
    }), {
        target: getZodFieldRule(entityParamMapService.entityParaMap.props),
    });
    return zod_1.z.object(fieldShape).strict('Should be only target of relation');
}
function zodFieldsQuery(entityParamMapService) {
    return getShapeFields(entityParamMapService)
        .refine((0, zod_utils_1.nonEmptyObject)(), {
        message: 'Validation error: Select target or relation fields',
    })
        .nullable();
}
//# sourceMappingURL=fields.js.map