"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.zodFilterQuery = zodFilterQuery;
const json_api_nestjs_shared_1 = require("@klerick/json-api-nestjs-shared");
const zod_1 = require("zod");
const types_1 = require("../../../../types");
const zod_utils_1 = require("../zod-utils");
const zodRuleForString = zod_1.z.union([
    zod_1.z.literal('null').transform(() => null),
    zod_1.z.string().refine((0, zod_utils_1.stringLongerThan)(), {
        message: 'String should be not empty',
    }),
]);
const zodRuleStringArray = zodRuleForString
    .array()
    .nonempty()
    .refine((0, zod_utils_1.arrayItemStringLongerThan)(0), {
    message: 'Array should be not empty',
});
const zodNullRule = zod_1.z.union([
    zod_1.z.literal('null'),
    zod_1.z.literal(null).transform((r) => 'null'),
]);
const zodRuleForArrayField = zod_1.z
    .object({ [json_api_nestjs_shared_1.FilterOperand.some]: zodRuleStringArray })
    .strict();
const zodRuleFilterRelationSchema = zod_1.z.union([
    zod_1.z
        .object({
        [json_api_nestjs_shared_1.FilterOperand.eq]: zodNullRule,
    })
        .strict(),
    zod_1.z
        .object({
        [json_api_nestjs_shared_1.FilterOperand.ne]: zodNullRule,
    })
        .strict(),
]);
function getZodRulesForField(type = types_1.TypeField.string) {
    const simpleShape = json_api_nestjs_shared_1.ObjectTyped.entries(json_api_nestjs_shared_1.FilterOperandOnlySimple).reduce((acum, [key, val]) => ({
        ...acum,
        [val]: zodRuleForString
            .refine((0, zod_utils_1.stringMustBe)(type), {
            message: `String should be as ${type}`,
        })
            .optional(),
    }), {});
    const ninInShape = json_api_nestjs_shared_1.ObjectTyped.entries(json_api_nestjs_shared_1.FilterOperandOnlyInNin).reduce((acum, [key, val]) => ({
        ...acum,
        [val]: zodRuleStringArray
            .refine((0, zod_utils_1.elementOfArrayMustBe)(type), {
            message: `String should be as ${type}`,
        })
            .optional(),
    }), {});
    return zod_1.z
        .object({
        ...simpleShape,
        ...ninInShape,
    })
        .strict()
        .refine((0, zod_utils_1.oneOf)(Object.values(json_api_nestjs_shared_1.FilterOperand).filter((i) => i !== json_api_nestjs_shared_1.FilterOperand.some)), {
        message: `Must have one of: "${Object.values(json_api_nestjs_shared_1.FilterOperand)
            .filter((i) => i !== json_api_nestjs_shared_1.FilterOperand.some)
            .join('","')}"`,
    });
}
function getZodRulesForRelationShape(shape) {
    return zod_1.z.object(shape).strict().optional().refine((0, zod_utils_1.nonEmptyObject)());
}
function getFilterPropsShapeForEntity(entityParam) {
    return entityParam.props.reduce((acum, field) => {
        const propertyType = Reflect.get(entityParam.propsType, field);
        const value = propertyType === types_1.TypeField.array
            ? zodRuleForArrayField
            : getZodRulesForField(propertyType);
        Reflect.set(acum, field, value.optional());
        return acum;
    }, {});
}
function zodFilterQuery(entityParamMapService) {
    const targetResult = entityParamMapService.entityParaMap.relations.reduce((acum, item) => ({
        ...acum,
        [item]: zodRuleFilterRelationSchema.optional(),
    }), getFilterPropsShapeForEntity(entityParamMapService.entityParaMap));
    const relationFilterProps = json_api_nestjs_shared_1.ObjectTyped.entries(entityParamMapService.entityParaMap.relationProperty).reduce((acum, [relName, relProps]) => {
        const relParams = entityParamMapService.getParamMap(relProps.entityClass);
        const zodFilter = getZodRulesForRelationShape(getFilterPropsShapeForEntity(relParams));
        return {
            ...acum,
            [relName]: zodFilter.optional(),
        };
    }, {});
    const targetShapeFilter = {
        target: zod_1.z
            .object(targetResult)
            .strict()
            .optional()
            .refine((0, zod_utils_1.nonEmptyObject)())
            .nullable(),
        relation: zod_1.z
            .object(relationFilterProps)
            .strict()
            .optional()
            .refine((0, zod_utils_1.nonEmptyObject)())
            .nullable(),
    };
    return zod_1.z.object(targetShapeFilter).strict().refine((0, zod_utils_1.nonEmptyObject)());
}
//# sourceMappingURL=filter.js.map