import { z, ZodArray } from 'zod';
import { EntityParam, TypeField } from '../../../../types';
import { ResultSchema } from './type';
import { EntityParamMapService } from '../../service';
declare const literalSchema: z.ZodUnion<[z.ZodString, z.ZodNumber, z.ZodBoolean, z.ZodNull]>;
type Literal = z.infer<typeof literalSchema>;
type Json = Literal | {
    [key: string]: Json;
} | Json[];
export declare function getZodRulesForNumber<Null extends true | false, isPatch extends true | false>(isNullable: Null, isPatch: isPatch): ResultSchema<z.ZodEffects<z.ZodNumber, number, unknown>, Null, isPatch>;
declare function getZodRulesForString<Null extends true | false, isPatch extends true | false>(isNullable: Null, isPatch: isPatch): ResultSchema<z.ZodString, Null, isPatch>;
declare function getZodRulesForDate<Null extends true | false, isPatch extends true | false>(isNullable: Null, isPatch: isPatch): ResultSchema<z.ZodDate, Null, isPatch>;
declare function getZodRulesForBoolean<Null extends true | false, isPatch extends true | false>(isNullable: Null, isPatch: isPatch): ResultSchema<z.ZodBoolean, Null, isPatch>;
declare function getZodSchemaForJson<Null extends true | false, isPatch extends true | false>(isNullable: Null, isPatch: isPatch): ResultSchema<z.ZodType<Json, z.ZodTypeDef, Json>, Null, isPatch>;
type ZodRulesResultArray<T extends TypeField, Null extends true | false, isPatch extends true | false> = ResultSchema<ZodArray<ZodRulesForType<T, false, false>, 'many'>, Null, isPatch>;
declare function getZodRulesForArray<T extends TypeField, Null extends true | false, isPatch extends true | false>(propsField: T, isNullable: Null, isPatch: isPatch): ZodRulesResultArray<T, Null, isPatch>;
type ZodRulesForArray<T extends TypeField, Null extends true | false, isPatch extends true | false> = ReturnType<typeof getZodRulesForArray<T, Null, isPatch>>;
export type ZodRulesForType<T extends TypeField, Null extends true | false, isPatch extends true | false> = ReturnType<T extends TypeField.date ? typeof getZodRulesForDate<Null, isPatch> : T extends TypeField.boolean ? typeof getZodRulesForBoolean<Null, isPatch> : T extends TypeField.number ? typeof getZodRulesForNumber<Null, isPatch> : T extends TypeField.object ? typeof getZodSchemaForJson<Null, isPatch> : typeof getZodRulesForString<Null, isPatch>>;
export type IsNullableProps<E extends object, IdKey extends string, K extends keyof E> = K extends EntityParam<E, IdKey>['propsNullable'][number] ? true : false;
type IsTypeArray<T extends TypeField> = T extends TypeField.array ? 1 : 0;
type Props<E extends object, IdKey extends string> = EntityParam<E, IdKey>['propsType'];
type PropsArray<E extends object, IdKey extends string> = EntityParam<E, IdKey>['propsArrayType'];
type OmitPrimary<E extends object, IdKey extends string> = Omit<Props<E, IdKey>, IdKey>;
type ZodRules<E extends object, IdKey extends string, P extends keyof E, IsArray extends 1 | 0, isPatch extends boolean> = IsArray extends 1 ? P extends keyof PropsArray<E, IdKey> ? ZodRulesForArray<PropsArray<E, IdKey>[P], IsNullableProps<E, IdKey, P>, isPatch> : never : P extends keyof Props<E, IdKey> ? ZodRulesForType<Props<E, IdKey>[P], IsNullableProps<E, IdKey, P>, isPatch> : never;
export type ShapeAttributesType<E extends object, IdKey extends string, isPatch extends boolean> = {
    [K in keyof OmitPrimary<E, IdKey>]: ZodRules<E, IdKey, K, IsTypeArray<OmitPrimary<E, IdKey>[K]>, isPatch>;
};
export declare function zodAttributes<E extends object, IdKey extends string, S extends true | false = false>(entityParamMapService: EntityParamMapService<E, IdKey>, isPatch: S): z.ZodEffects<z.ZodObject<ShapeAttributesType<E, IdKey, S>, "strict", z.ZodTypeAny, z.objectUtil.addQuestionMarks<z.baseObjectOutputType<ShapeAttributesType<E, IdKey, S>>, any> extends infer T ? { [k in keyof T]: z.objectUtil.addQuestionMarks<z.baseObjectOutputType<ShapeAttributesType<E, IdKey, S>>, any>[k]; } : never, z.baseObjectInputType<ShapeAttributesType<E, IdKey, S>> extends infer T_1 ? { [k_1 in keyof T_1]: z.baseObjectInputType<ShapeAttributesType<E, IdKey, S>>[k_1]; } : never>, z.objectUtil.addQuestionMarks<z.baseObjectOutputType<ShapeAttributesType<E, IdKey, S>>, any> extends infer T_2 ? { [k in keyof T_2]: z.objectUtil.addQuestionMarks<z.baseObjectOutputType<ShapeAttributesType<E, IdKey, S>>, any>[k]; } : never, z.baseObjectInputType<ShapeAttributesType<E, IdKey, S>> extends infer T_3 ? { [k_1 in keyof T_3]: z.baseObjectInputType<ShapeAttributesType<E, IdKey, S>>[k_1]; } : never>;
export type ZodAttributes<E extends object, IdKey extends string, K extends true | false = false> = ReturnType<typeof zodAttributes<E, IdKey, K>>;
export type Attributes<E extends object, IdKey extends string, K extends true | false = false> = z.infer<ZodAttributes<E, IdKey, K>>;
export {};
